﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Labyrinth
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            const int höhe = 20;
            const int breite = 30;
            const double raster = 10.0;
            Random random = new Random();

            bool[,] tunnelOberhalb = new bool[höhe+1, breite];
            bool[,] tunnelLinks = new bool[höhe, breite+1];

            tunnelOberhalb[2, 3] = true;

            for (int i = 0; i < 30000; i++)
            {
                // Münzwurf -> horizontal oder vertikal
                if(random.Next(2) == 0) // horizontal
                {
                    // Zufallszahl -> ein theoretischer Tunnel
                    int x = random.Next(breite);
                    int y = random.Next(höhe + 1);

                    // Ist Tunnel bereits gebaut?
                    if(tunnelOberhalb[y,x])
                    {
                        continue;
                    }

                    // Grenzt Tunnel an bisherige an?
                    // links/rechts
                    bool linksVerbunden = false;

                    if(y > 0 && tunnelLinks[y - 1, x])
                    { linksVerbunden = true;  }
                    else if (x > 0 && tunnelOberhalb[y, x - 1])
                    { linksVerbunden = true; }
                    else if (y < höhe && tunnelLinks[y, x])
                    { linksVerbunden = true; }

                    bool rechtsVerbunden = false;

                    if (x < breite && y > 0 && tunnelLinks[y - 1, x + 1])
                    { rechtsVerbunden = true; }
                    else if (x + 1 < breite && tunnelOberhalb[y, x + 1])
                    { rechtsVerbunden = true; }
                    else if (x < breite && y < höhe && tunnelLinks[y, x + 1])
                    { rechtsVerbunden = true; }

                    if(linksVerbunden ^ rechtsVerbunden)
                    // also mit bisherigen verbunden und kein Kurzschluss
                    {
                        tunnelOberhalb[y, x] = true;
                    }
                }
                else // vertikal
                {
                    // Zufallszahl -> ein theoretischer Tunnel
                    int x = random.Next(breite + 1);
                    int y = random.Next(höhe);

                    // Ist Tunnel bereits gebaut?
                    if (tunnelLinks[y, x])
                    {
                        continue;
                    }

                    // Grenzt Tunnel an bisherige an?
                    // links/rechts
                    bool obenVerbunden = false;

                    if (x < breite && tunnelOberhalb[y, x])
                    { obenVerbunden = true; }
                    else if (y > 0 && tunnelLinks[y - 1, x])
                    { obenVerbunden = true; }
                    else if (x > 0 && tunnelOberhalb[y, x - 1])
                    { obenVerbunden = true; }

                    bool untenVerbunden = false;

                    if (x < breite && y < höhe && tunnelOberhalb[y + 1, x])
                    { untenVerbunden = true; }
                    else if (y + 1 < höhe && tunnelLinks[y + 1, x])
                    { untenVerbunden = true; }
                    else if (x > 0 && y < höhe && tunnelOberhalb[y + 1, x - 1])
                    { untenVerbunden = true; }

                    if (obenVerbunden ^ untenVerbunden)
                    // also mit bisherigen verbunden und kein Kurzschluss
                    {
                        tunnelLinks[y, x] = true;
                    }
                }
            }

            for (int y = 0; y < höhe + 1; y++)
            {
                for (int x = 0; x < breite; x++)
                {
                    if(tunnelOberhalb[y, x])
                    {
                        Line a = new Line();
                        a.X1 = (x + 1) * raster;
                        a.Y1 = (y + 1) * raster;
                        a.X2 = (x + 2) * raster;
                        a.Y2 = (y + 1) * raster;
                        a.Stroke = Brushes.White;
                        a.StrokeThickness = 5.0;
                        a.StrokeStartLineCap = PenLineCap.Square;
                        a.StrokeEndLineCap = PenLineCap.Square;
                        zeichenfläche.Children.Add(a);
                    }
                }
            }

            for (int y = 0; y < höhe; y++)
            {
                for (int x = 0; x < breite + 1; x++)
                {
                    if (tunnelLinks[y, x])
                    {
                        Line a = new Line();
                        a.X1 = (x + 1) * raster;
                        a.Y1 = (y + 1) * raster;
                        a.X2 = (x + 1) * raster;
                        a.Y2 = (y + 2) * raster;
                        a.Stroke = Brushes.White;
                        a.StrokeThickness = 5.0;
                        a.StrokeStartLineCap = PenLineCap.Square;
                        a.StrokeEndLineCap = PenLineCap.Square;
                        zeichenfläche.Children.Add(a);
                    }
                }
            }
        }
    }
}
